@@ -16,8 +16,11 @@ sub match {
     my $regex = qr{(?>
          access[ ]denied[.][ ]IP[ ]name[ ]lookup[ ]failed
         |access[ ]from[ ]ip[ ]address[ ].+[ ]blocked
+        |blacklisted[ ]by
         |Blocked[ ]-[ ]see[ ]https://support[.]proofpoint[.]com/dnsbl-lookup[.]cgi[?]ip=.+
+        |can[']t[ ]determine[ ]Purported[ ]Responsible[ ]Address
         |cannot[ ]resolve[ ]your[ ]address
+        |client[ ]host[ ].+[ ]blocked[ ]using
         |client[ ]host[ ]rejected:[ ](?:
              may[ ]not[ ]be[ ]mail[ ]exchanger
             |cannot[ ]find[ ]your[ ]hostname    # Yahoo!
@@ -57,6 +60,7 @@ sub match {
              network[ ]is[ ]temporary[ ]blacklisted
             |server[ ]requires[ ]confirmation
             )
+        |was[ ]blocked[ ]by[ ].+
         |we[ ]do[ ]not[ ]accept[ ]mail[ ]from[ ](?: # @mail.ru
              hosts[ ]with[ ]dynamic[ ]IP[ ]or[ ]generic[ ]dns[ ]PTR-records
             |dynamic[ ]ips
@@ -152,7 +156,7 @@ azumakuniyuki
 
 =head1 COPYRIGHT
 
-Copyright (C) 2014-2016 azumakuniyuki, All rights reserved.
+Copyright (C) 2014-2017 azumakuniyuki, All rights reserved.
 
 =head1 LICENSE
 
